#
#  DCfunction-attributeSUM.rb
#  
#
#  Created by Tom Kaneko on 10/06/2013.
#  Copyright (c) 2013 __MyCompanyName__. All rights reserved.
#
require 'sketchup.rb'
require 'extensions.rb'
require 'dynamiccomponents.rb'


if defined?($dc_observers)
	# Open SketchUp's Dynamic Component Functions (V1) class.
	# only if DC extension is active
	
	UI.messagebox "loaded"
	
	# Skip any entity that is not a group or component when looking for attributes
	SKIP_NON_GROUPS = true
	
	class DCFunctionsV1
		protected
	
		# return attributesum
		# Usage: =attributesum("attribute_name", "dict_name"[optional])
		def attributesum(a)
			puts 'called once.  source_entity:'
			puts @source_entity
			# Check attribute_name has been passed to the function
			if a[0].nil? || a[0] == 0
				UI.messagebox('You need to specify an attribute name e.g. attributesum("weight")')
				return false
			end #if
	
			attribute_name = a[0]
			
			# Replace with default Dictionary Name if empty
			if defined? a[1] && (a[1].nil? || a[1] == 0)
				dict_name = 'dynamic_attributes'
			else
				dict_name = a[1]
			end #if
	
			#load the helper class
			helper = AttributesFormulaHelper.new

			sum = 0
			
			# Get the total sum of all child elements with this attribute_name
			sum += helper.get_sum_of_entities(@source_entity, attribute_name, dict_name)
	
			sum
		end #attributesum
		protected:attributesum

	end#class



	class AttributesFormulaHelper
	
		# return sum of attribute values with attribute_name in entity
		def get_sum_of_entities(entity, attribute_name, dict_name)
		
			return 0 if (SKIP_NON_GROUPS && !(entity.is_a? Sketchup::Group) && !(entity.is_a? Sketchup::ComponentInstance))
			
			sum = 0
		
			value = entity.get_attribute(dict_name, attribute_name, false)
		
			if value && value.numeric?
				sum += value.to_f
			end #if

			if entity.typename == 'Group'
			
				# get any child entites which are groups or components
				entity.entities.each do |ent|
					sum += get_sum_of_entities(ent, attribute_name, dict_name)
				end 
			elsif entity.typename == 'ComponentInstance'
			
				entity.definition.entities.each do |ent|
					sum += get_sum_of_entities(ent, attribute_name, dict_name)
				end
			end
		
			sum
		end #def
	
	end #class

	# Observer class for tying changes in attributes
	class AttributeEntityObserver < Sketchup::EntityObserver
		def onEraseEntity(entity)
			puts 'Erasing' + entity.to_s
		end
		
		def onChangeEntity(entity)
			puts 'onChangeEntity' + entity.to_s
		end
	end

	class String
		def numeric?
			Float(self) != nil rescue false
		end
	end

end#if

file_loaded("DCfunction-attributeSUM.rb")   # Mark the script loaded. 